//
//  RCTAMapManager.m
//  RCTAMap
//
//  Created by 王继荣 on 20/07/2017.
//  Copyright © 2017 cn.wonderbear. All rights reserved.
//

#import "AMapManager.h"

@interface AMapManager()
{
    BOOL isReCenter;
    MAMapView *mapView;
    CLLocationManager *locationManager;
    AMapEventEmitter *eventEmitter;
    AMapNaviManager *naviManager;
}

@end

static AMapManager *sharedInstance;

@implementation AMapManager

RCT_EXPORT_MODULE()

+ (id)allocWithZone:(struct _NSZone *)zone
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [super allocWithZone:zone];
    });
    return sharedInstance;
}

+ (instancetype)sharedInstance
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [self new];
    });
    return sharedInstance;
}

- (id)copyWithZone:(NSZone *)zone
{
    return sharedInstance;
}

- (dispatch_queue_t)methodQueue
{
    return self.bridge.uiManager.methodQueue;
}

- (UIView *)view
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        [AMapServices sharedServices].enableHTTPS = YES;
        NSString *plistPath = [[NSBundle mainBundle] pathForResource:@"Info" ofType:@"plist"];  
        NSMutableDictionary *dict = [[NSMutableDictionary alloc] initWithContentsOfFile:plistPath];
        NSString* apiKey = [ dict objectForKey:@"AMapApiKey" ];
        [AMapServices sharedServices].apiKey = apiKey;
    });
    
    if(![CLLocationManager locationServicesEnabled]){
        NSLog(@"请开启定位:设置 > 隐私 > 位置 > 定位服务");
    }
    
    if (eventEmitter == nil) {
        eventEmitter = [AMapEventEmitter new];
    }
    
    if (naviManager == nil) {
        naviManager = [AMapNaviManager new];
    }
    
    if (locationManager == nil) {
        locationManager = [CLLocationManager new];
        locationManager.delegate = self;
        locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        [locationManager requestWhenInUseAuthorization];
        [locationManager startUpdatingLocation];
    }
    
    if (mapView == nil) {
        mapView = [MAMapView new];
        mapView.delegate = eventEmitter;
        mapView.showsUserLocation = YES;
        mapView.showTraffic = YES;
        mapView.showsCompass = NO;
        mapView.userTrackingMode = MAUserTrackingModeFollowWithHeading;
        [mapView setCenterCoordinate:CLLocationCoordinate2DMake(38.490462, 106.237214)];
    }
    
    isReCenter = true;
    [mapView setZoomLevel:11];
    [mapView removeAnnotations:[mapView annotations]];
    
    return mapView;
}

- (void)dealloc {
    mapView = nil;
    locationManager = nil;
    eventEmitter = nil;
}

RCT_EXPORT_METHOD(setCenterCoordinate: (nonnull NSNumber *)reactTag withCoordinate: (CLLocationCoordinate2D)coordinate animated: (BOOL)animated)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap setCenterCoordinate:coordinate animated:animated];
        }
    }];
}

RCT_EXPORT_METHOD(reCenter: (nonnull NSNumber *)reactTag)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap setCenterCoordinate:aMap.userLocation.coordinate animated:true];
        }
    }];
}

RCT_EXPORT_METHOD(reloadMap: (nonnull NSNumber *)reactTag)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap reloadMap];
        }
    }];
}

RCT_EXPORT_METHOD(showAnnotations: (nonnull NSNumber *)reactTag withAnnotations: (nonnull NSArray<AMapAnnotation *> *)annotations)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap showAnnotations:annotations animated:true];
        }
    }];
}

RCT_EXPORT_METHOD(addAnnotation: (nonnull NSNumber *)reactTag withAnnotation: (nonnull AMapAnnotation *)annotation)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            annotation.title = @"hello";
            annotation.subtitle = @"world";
            [aMap addAnnotation:annotation];
        }
    }];
}

RCT_EXPORT_METHOD(addAnnotations: (nonnull NSNumber *)reactTag withAnnotations: (nonnull NSArray<AMapAnnotation *> *)annotations)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap addAnnotations:annotations];
        }
    }];
}

RCT_EXPORT_METHOD(removeAnnotation: (nonnull NSNumber *)reactTag withAnnotation: (nonnull AMapAnnotation *)annotation)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap removeAnnotation:annotation];
        }
    }];
}

RCT_EXPORT_METHOD(removeAnnotations: (nonnull NSNumber *)reactTag withAnnotations: (nonnull NSArray<AMapAnnotation *> *)annotations)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap removeAnnotations:annotations];
        }
    }];
}

RCT_EXPORT_METHOD(clearAnnotations: (nonnull NSNumber *)reactTag)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap removeAnnotations:[aMap annotations]];
        }
    }];
}

RCT_EXPORT_METHOD(selectAnnotation: (nonnull NSNumber *)reactTag withAnnotation: (nonnull AMapAnnotation *)annotation)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap selectAnnotation:annotation animated:true];
        }
    }];
}

RCT_EXPORT_METHOD(deselectAnnotation: (nonnull NSNumber *)reactTag withAnnotation: (nonnull AMapAnnotation *)annotation)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap deselectAnnotation:annotation animated:true];
        }
    }];
}

RCT_EXPORT_METHOD(naviTo: (nonnull NSNumber *)reactTag destination: (nonnull AMapNaviEndPoint*)destination) 
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            [naviManager naviTo:destination];
        }
    }];
}

RCT_EXPORT_METHOD(naviFromTo: (nonnull NSNumber *)reactTag from: (nonnull AMapNaviEndPoint*)source  to:(nonnull AMapNaviEndPoint*)destination) 
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            [naviManager naviFrom:source to:destination];
        }
    }];
}

RCT_EXPORT_METHOD(setVisibleMapRect: (nonnull NSNumber *)reactTag withRect: (MAMapRect)mapRect animated: (BOOL)animated)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        UIView *view = viewRegistry[reactTag];
        if (![view isKindOfClass:[MAMapView class]]) {
            RCTLog(@"expecting MAMapView, got: %@", view);
        }
        else {
            MAMapView* aMap = (MAMapView*)view;
            [aMap setVisibleMapRect:mapRect animated:animated];
        }
    }];
}

#pragma mark - CLLocationManagerDelegate

- (void)locationManager:(CLLocationManager *)manager
    didUpdateToLocation:(CLLocation *)newLocation
           fromLocation:(CLLocation *)oldLocation
{
    if (isReCenter) {
        isReCenter = false;
        [mapView setCenterCoordinate:newLocation.coordinate animated:true];
    }
}

- (void)locationManager:(CLLocationManager *)manager
       didFailWithError:(NSError *)error
{
    //定位失败
    
}

@end

