//
//  AMapModule.m
//  RCTAMap
//
//  Created by 王继荣 on 21/09/2017.
//  Copyright © 2017 cn.wonderbear. All rights reserved.
//

#import "AMapModule.h"
#import <AMapFoundationKit/AMapFoundationKit.h>
#import <AMapLocationKit/AMapLocationKit.h>

@interface AMapModule() <AMapLocationManagerDelegate>
{
    AMapLocationManager *locationManager;
}

@end

@implementation AMapModule

RCT_EXPORT_MODULE();

RCT_EXPORT_METHOD(getLocation:(RCTPromiseResolveBlock)resolve
                  withRejecter:(RCTPromiseRejectBlock)reject)
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        [AMapServices sharedServices].enableHTTPS = YES;
        NSString *plistPath = [[NSBundle mainBundle] pathForResource:@"Info" ofType:@"plist"];  
        NSMutableDictionary *dict = [[NSMutableDictionary alloc] initWithContentsOfFile:plistPath];
        NSString* apiKey = [ dict objectForKey:@"AMapApiKey" ];
        [AMapServices sharedServices].apiKey = apiKey;
        
        locationManager = [[AMapLocationManager alloc] init];
        locationManager.delegate = self;
    });
    
    // 带逆地理信息的一次定位（返回坐标和地址信息）
    [locationManager setDesiredAccuracy:kCLLocationAccuracyHundredMeters];
    //   定位超时时间，最低2s，此处设置为2s
    locationManager.locationTimeout =2;
    //   逆地理请求超时时间，最低2s，此处设置为2s
    locationManager.reGeocodeTimeout = 2;
    
    // 带逆地理（返回坐标和地址信息）。将下面代码中的 YES 改成 NO ，则不会返回地址信息。
    [locationManager requestLocationWithReGeocode:YES completionBlock:^(CLLocation *location, AMapLocationReGeocode *regeocode, NSError *error) {
        
        if (error)
        {
            NSLog(@"locError:{%ld - %@};", (long)error.code, error.localizedDescription);
            reject([NSString stringWithFormat:@"%ld", (long)error.code], error.localizedDescription, error);
            return;
        }
        
        NSLog(@"location:%@", location);
        resolve([NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithDouble:location.coordinate.longitude], @"longitude", [NSNumber numberWithDouble:location.coordinate.latitude], @"latitude", nil]);
        
        if (regeocode)
        {
            NSLog(@"reGeocode:%@", regeocode);
        }
    }];
}

- (void)amapLocationManager:(AMapLocationManager *)manager didUpdateLocation:(CLLocation *)location reGeocode:(AMapLocationReGeocode *)reGeocode
{
    NSLog(@"location:{lat:%f; lon:%f; accuracy:%f}", location.coordinate.latitude, location.coordinate.longitude, location.horizontalAccuracy);
    if (reGeocode)
    {
        NSLog(@"reGeocode:%@", reGeocode);
    }
}

@end
