//
//  AMapNaviManager.m
//  RCTAMap
//
//  Created by 王继荣 on 21/08/2017.
//  Copyright © 2017 cn.wonderbear. All rights reserved.
//

#import "AMapNaviManager.h"

@interface AMapNaviManager()
{
    AMapNaviCompositeManager* compositeManager;
    AMapEventEmitter *eventEmitter;
}

@end

static AMapNaviManager *sharedInstance;

@implementation AMapNaviManager

RCT_EXPORT_MODULE();

+ (id)allocWithZone:(struct _NSZone *)zone
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [super allocWithZone:zone];
    });
    return sharedInstance;
}

+ (instancetype)sharedInstance
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [self new];
    });
    return sharedInstance;
}

- (id)copyWithZone:(NSZone *)zone
{
    return sharedInstance;
}

- (instancetype)init {
    if (sharedInstance == nil) {
        sharedInstance = [super init];
    }
    self = sharedInstance;
    if (self) {
        compositeManager = [AMapNaviCompositeManager new];
        if (eventEmitter == nil) {
            eventEmitter = [AMapEventEmitter new];
            compositeManager.delegate = eventEmitter;
        }
    }
    return self;
}

- (void)naviTo:(id <MAEndPoint>)destination {
    AMapNaviCompositeUserConfig *config = [AMapNaviCompositeUserConfig new];
    AMapNaviPoint* location = [AMapNaviPoint locationWithLatitude:destination.location.latitude longitude:destination.location.longitude];
    [config setRoutePlanPOIType:AMapNaviRoutePlanPOITypeEnd location:location name:destination.name POIId:destination.mid];
    [compositeManager presentRoutePlanViewControllerWithOptions:config];
}

- (void)naviFrom:(id <MAEndPoint>)source to:(id <MAEndPoint>)destination {
    AMapNaviCompositeUserConfig *config = [AMapNaviCompositeUserConfig new];
    AMapNaviPoint* sourceLocation = [AMapNaviPoint locationWithLatitude:source.location.latitude longitude:source.location.longitude];
    [config setRoutePlanPOIType:AMapNaviRoutePlanPOITypeStart location:sourceLocation name:source.name POIId:source.mid];
    AMapNaviPoint* destinationLocation = [AMapNaviPoint locationWithLatitude:destination.location.latitude longitude:destination.location.longitude];
    [config setRoutePlanPOIType:AMapNaviRoutePlanPOITypeEnd location:destinationLocation name:destination.name POIId:destination.mid];
    [compositeManager presentRoutePlanViewControllerWithOptions:config];
}

@end
